#include "MotionData.h"


CKeyFrameMotion::CKeyFrameMotion()
{
	return;
}
CKeyFrameMotion::~CKeyFrameMotion()
{
	init();
}

void CKeyFrameMotion::init(void)
{
	qArray.clear();
	frameArray.clear();
	interTypeArray.clear();
	basePosArray.clear();
	objSwapEvents.clear();

	return;
}

fk_Vector CKeyFrameMotion::getInterVec(const fk_Vector &vecA, const fk_Vector &vecB, double t, fk_MotionInterType argType)
{
	if(vecA == vecB) return vecA;

	switch(argType) {
	case LINEAR:
		return (1.0-t)*vecA + t*vecB;
	case SPHERE:
		return pow(cos(t*FK_PI/2.0), 2)*vecA + pow(sin(t*FK_PI/2.0), 2)*vecB;
	case POWER:
		return (1.0-pow(t, 3))*vecA + pow(t, 3)*vecB;
	case REV_POWER:
		return pow(1.0-t, 3)*vecA + (1.0-pow(1.0-t, 3))*vecB;
	case FREEZE:
	case HIDE:		// <----------------------------------------------------IuWFNĝւΉ̂ߒǉ
		return vecB;
	default:
		break;
	}

	return fk_Vector();
}

fk_Quaternion CKeyFrameMotion::getInterQ(const fk_Quaternion &qA, const fk_Quaternion &qB, double t, fk_MotionInterType argType)
{
	if(qA == qB) return qA;

	switch(argType) {
	case LINEAR:
	case SPHERE:
		return fk_Math::quatInterSphere(qA, qB, t);
	case POWER:
		return fk_Math::quatInterSphere(qA, qB, pow(t, 3));
	case REV_POWER:
		return fk_Math::quatInterSphere(qA, qB, (1.0-pow(1.0-t, 3)));
	case FREEZE:
	case HIDE:		// <----------------------------------------------------IuWFNĝւΉ̂ߒǉ
		return qB;
	default:
		return fk_Quaternion();
	}
}

void CKeyFrameMotion::pushKeyFrame(const fk_Quaternion &argQ, int argFrame, fk_MotionInterType argType, int argKeyPos)
{
	if(argKeyPos == -1) {
		qArray.push_back(argQ);
		frameArray.push_back(argFrame);
		interTypeArray.push_back(argType);
	} else if(argKeyPos < (int)qArray.size()){
		qArray.insert(qArray.begin()+argKeyPos, argQ);
		frameArray.insert(frameArray.begin()+argKeyPos, argFrame);
		interTypeArray.insert(interTypeArray.begin()+argKeyPos, argType);
	}

	return;
}
void CKeyFrameMotion::pushKeyFrame(const fk_Quaternion &argQ, const fk_Vector &argBasePos, int argFrame, fk_MotionInterType argType, int argKeyPos)
{
	if(argKeyPos == -1) {
		qArray.push_back(argQ);
		frameArray.push_back(argFrame);
		interTypeArray.push_back(argType);
		basePosArray.push_back(argBasePos);
	} else if(argKeyPos < (int)qArray.size()){
		qArray.insert(qArray.begin()+argKeyPos, argQ);
		frameArray.insert(frameArray.begin()+argKeyPos, argFrame);
		interTypeArray.insert(interTypeArray.begin()+argKeyPos, argType);
		basePosArray.insert(basePosArray.begin()+argKeyPos, argBasePos);
	}
	return;
}
bool CKeyFrameMotion::changeKeyFrame(int argKeyID, const fk_Quaternion &argQ, int argFrame, fk_MotionInterType argType)
{
	if(argKeyID >= getTotalKeyNum()) return false;

	qArray[argKeyID] = argQ;
	frameArray[argKeyID] = argFrame;
	interTypeArray[argKeyID] = argType;

	return true;
}
bool CKeyFrameMotion::changeKeyFrame(int argKeyID, const fk_Quaternion &argQ, const fk_Vector &argBasePos, int argFrame, fk_MotionInterType argType)
{
	if(argKeyID >= getTotalKeyNum()) return false;

	qArray[argKeyID] = argQ;
	frameArray[argKeyID] = argFrame;
	interTypeArray[argKeyID] = argType;
	basePosArray[argKeyID] = argBasePos;

	return true;
}
bool CKeyFrameMotion::deleteKeyFrame(int argKeyID)
{
	if(argKeyID >= getTotalKeyNum()) return false;

	qArray.erase(qArray.begin()+argKeyID);
	frameArray.erase(frameArray.begin()+argKeyID);
	interTypeArray.erase(interTypeArray.begin()+argKeyID);

	if(isBaseMotion()) basePosArray.erase(basePosArray.begin()+argKeyID);

	return true;
}

bool CKeyFrameMotion::isBaseMotion(void)
{
	if(basePosArray.size() != 0) return true;
	else return false;
}
int CKeyFrameMotion::getTotalKeyNum(void)
{
	return (int)qArray.size();
}

int CKeyFrameMotion::getTotalFrameNum(void)
{
	int retVal = 0;

	for(int i = 0; i < getTotalKeyNum(); i++) {
		retVal += frameArray[i];
	}

	return retVal;
}
bool CKeyFrameMotion::getFrameToKeyID(int argFrame, int *outID, int *outFrame)
{
	if(argFrame >= getTotalFrameNum()) return false;

	int	nowKey = 0, nowFrame = 0;
	for(int i = 0; i < getTotalKeyNum(); i++) {
		nowKey = i;
		nowFrame += frameArray[i];
		if(argFrame < nowFrame) break;
	}

	*outID = nowKey;
	*outFrame = argFrame-(nowFrame-frameArray[nowKey]);

	return true;
}

void CKeyFrameMotion::setFrameState(int argFrame, fk_Model *argModel)
{
	static int				nowKey = 0, stepFrame = 0;
	static fk_Quaternion	q;
	const fk_Vector			dirV(0.0, 0.0, -1.0);
	const fk_Vector			upV(0.0, 1.0, 0.0);
	
	if(!getFrameToKeyID(argFrame, &nowKey, &stepFrame)) return;

	if(nowKey != 0 && stepFrame == 0) {
		argModel->glVec(qArray[nowKey-1]*dirV);
		argModel->glUpvec(qArray[nowKey-1]*upV);

		if(isBaseMotion()) {
			argModel->glMoveTo(basePosArray[nowKey-1]);//-basePosArray[0]);
		}
	} else if(nowKey != 0) {
       	q = getInterQ(qArray[nowKey-1], qArray[nowKey], (double)(stepFrame+1)/(double)frameArray[nowKey], interTypeArray[nowKey]);
		argModel->glVec(q*dirV);
		argModel->glUpvec(q*upV);

		if(isBaseMotion()) {
			argModel->glMoveTo(getInterVec(basePosArray[nowKey-1], basePosArray[nowKey],
				(double)(stepFrame+1)/(double)frameArray[nowKey], interTypeArray[nowKey]));//-basePosArray[0]);
		}
	} else {
		static fk_Quaternion nowQ;
		nowQ.makeEuler(argModel->getAngle());

       	q = getInterQ(nowQ, qArray[nowKey], (double)(stepFrame+1)/(double)frameArray[nowKey]);
		argModel->glVec(q*dirV);
		argModel->glUpvec(q*upV);

		if(isBaseMotion()) {
			argModel->glMoveTo(getInterVec(argModel->getPosition(), basePosArray[nowKey],
				(double)(stepFrame+1)/(double)frameArray[nowKey]));//-basePosArray[0]);
		}
	}

	return;
}
